from collections import OrderedDict

import numpy as np
from OpenGL.GL import glDeleteTextures, glGenTextures, glTexImage2D
from OpenGL.raw.GL.ARB.internalformat_query2 import GL_TEXTURE_2D
from OpenGL.raw.GL.VERSION.GL_1_0 import GL_RGBA, GL_UNSIGNED_BYTE, glTexParameteri, GL_TEXTURE_MIN_FILTER, \
    GL_TEXTURE_MAG_FILTER, glPixelStorei, GL_UNPACK_ALIGNMENT, GL_LINEAR, GL_TEXTURE_WRAP_T, \
    GL_TEXTURE_WRAP_S
from OpenGL.raw.GL.VERSION.GL_1_1 import glBindTexture
from OpenGL.raw.GL.VERSION.GL_1_2 import GL_CLAMP_TO_EDGE


class TileManager:
    def __init__(self, deep_zoom):
        self.deep_zoom = deep_zoom
        self.cache = OrderedDict()

    def get_tile(self, level, tx, ty, tile_size):
        key = (level, tx, ty)

        if key in self.cache:
            self.cache.move_to_end(key)
            return self.cache[key]

        if len(self.cache) > 128:
            _, tex = self.cache.popitem(last=False)
            glDeleteTextures([tex])

        image = self.deep_zoom.levels[level]
        h, w = image.shape[:2]

        x0 = tx * tile_size
        y0 = ty * tile_size
        x1 = min(x0 + tile_size, w)
        y1 = min(y0 + tile_size, h)

        if x0 >= w or y0 >= h:
            return None

        tile = image[y0:y1, x0:x1]

        tile = tile.astype(np.uint8)
        if tile.shape[2] == 3:
            alpha = np.full((tile.shape[0], tile.shape[1], 1), 255, dtype=np.uint8)
            tile = np.concatenate([tile, alpha], axis=2)

        tex = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, tex)
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1)

        glTexImage2D(
            GL_TEXTURE_2D,
            0,
            GL_RGBA,
            tile.shape[1],
            tile.shape[0],
            0,
            GL_RGBA,
            GL_UNSIGNED_BYTE,
            tile
        )
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)

        self.cache[key] = tex
        return tex



from Imervue.multi_language.chinese import chinese_word_dict
from Imervue.multi_language.english import english_word_dict
from Imervue.multi_language.japanese import japanese_word_dict
from Imervue.multi_language.korean import korean_word_dict
from Imervue.multi_language.traditional_chinese import traditional_chinese_word_dict


class LanguageWrapper(object):

    def __init__(
            self
    ):
        self.language: str = "English"
        self.choose_language_dict = {
            "English": english_word_dict,
            "Traditional_Chinese": traditional_chinese_word_dict,
            "Chinese": chinese_word_dict,
            "Korean": korean_word_dict,
            "Japanese": japanese_word_dict,
        }
        self.language_word_dict: dict = self.choose_language_dict.get(self.language)

    def reset_language(self, language) -> None:
        if language in [
            "English",
            "Traditional_Chinese",
            "Chinese",
            "Korean",
            "Japanese",
        ]:
            self.language = language
            self.language_word_dict = self.choose_language_dict.get(self.language)


language_wrapper = LanguageWrapper()


korean_word_dict = {
    "main_window_current_filename": "현재 파일 이름:",
    "main_window_current_filename_format": "현재 파일 이름: {name}",
    "main_window_current_file": "파일",
    "main_window_open_folder": "폴더 열기",
    "main_window_exit": "종료",
    "main_window_tile_size": "썸네일 크기",
    "main_window_select_folder": "폴더 선택",
    "main_window_current_folder_format": "현재 폴더: {path}",
    "main_window_remove_undo_stack": "임시 되돌리기 파일 삭제",
    # Language Menubar
    "menu_bar_language": "언어",
    "language_menu_bar_please_restart_messagebox": "다시 시작해 주세요",
    "language_menu_bar_english": "영어",
    "language_menu_bar_traditional_chinese": "번체 중국어",
    "language_menu_bar_chinese": "간체 중국어",
    "language_menu_bar_koren": "한국어",
    "language_menu_bar_japanese": "일본어",
    # Tip menu
    "main_window_tip_menu": "사용 설명",
    "mouse_control_middle_tip": "마우스 휠 버튼으로 이동할 수 있습니다",
    "main_window_mouse_tip_menu": "마우스 조작",
    "main_window_keyboard_tip_menu": "키보드 조작",
    "mouse_control_left_tip": "마우스 왼쪽 버튼으로 이미지를 클릭하면 Deep zoom 모드로 들어갑니다",
    "mouse_control_right_move_tip": "마우스 오른쪽 버튼을 누른 상태에서 썸네일 모드에서 여러 이미지를 선택할 수 있습니다",
    "keyboard_control_esc_tip": "Deep zoom 모드에서는 ESC 키로 종료할 수 있습니다",
    "keyboard_control_tile_arrow_up_tip": "썸네일 모드에서는 방향키 위로 위쪽으로 이동할 수 있습니다",
    "keyboard_control_tile_arrow_down_tip": "썸네일 모드에서는 방향키 아래로 아래쪽으로 이동할 수 있습니다",
    "keyboard_control_tile_arrow_left_tip": "썸네일 모드에서는 방향키 왼쪽으로 왼쪽으로 이동할 수 있습니다",
    "keyboard_control_tile_arrow_right_tip": "썸네일 모드에서는 방향키 오른쪽으로 오른쪽으로 이동할 수 있습니다",
    "keyboard_control_delete_tip": "썸네일 모드와 Deep zoom 모드 모두에서 Delete 키로 이미지를 삭제할 수 있습니다",
}



import math


class DeepZoomImage:
    def __init__(self, image_array):
        self.levels = []
        self.build_pyramid(image_array)

    def build_pyramid(self, image):
        current = image
        self.levels.append(current)

        while min(current.shape[:2]) > 512:
            current = current[::2, ::2]
            self.levels.append(current)

    def get_level(self, zoom):
        # zoom 1.0 = full res
        level = int(max(0, min(len(self.levels)-1, -math.log2(zoom))))
        return level, self.levels[level]



import os
import sys
from pathlib import Path

from PySide6.QtCore import Qt, QTimer
from PySide6.QtGui import QActionGroup, QIcon
from PySide6.QtWidgets import (
    QApplication, QMainWindow, QFileDialog,
    QTreeView, QFileSystemModel, QSplitter, QSizePolicy
)
from PySide6.QtWidgets import QLabel, QVBoxLayout, QWidget

from Imervue.gpu_image_view.actions.delete import commit_pending_deletions
from Imervue.gpu_image_view.images.image_loader import load_image
from Imervue.menu.language_menu import build_language_menu
from Imervue.menu.tip_menu import build_tip_menu
from Imervue.multi_language.language_wrapper import language_wrapper
from Imervue.user_settings.user_setting_dict import write_user_setting, read_user_setting, user_setting_dict


class ImervueMainWindow(QMainWindow):
    def __init__(self, debug: bool = False):
        super().__init__()

        self.setWindowTitle("Imervue")

        # Windows 平台設定 AppUserModelID
        # Set AppUserModelID for Windows platform
        self.id = "Imervue"
        from ctypes import windll
        windll.shell32.SetCurrentProcessExplicitAppUserModelID(self.id)

        read_user_setting()

        # 語言支援
        # Language support
        self.language_wrapper = language_wrapper
        self.language_wrapper.reset_language(user_setting_dict.get("language", "English"))

        self.icon_path = Path(os.getcwd()) / "Imervue.ico"
        self.icon = QIcon(str(self.icon_path))
        self.setWindowIcon(self.icon)

        # ===== 中央 Splitter =====
        splitter = QSplitter()
        self.setCentralWidget(splitter)

        # ===== 左側檔案樹 =====
        self.model = QFileSystemModel()
        self.model.setRootPath("")

        self.tree = QTreeView()
        self.tree.setModel(self.model)
        self.tree.setRootIndex(self.model.index(""))
        self.model.setNameFilters(["*.png", "*.jpg", "*.jpeg", "*.bmp", "*.tiff"])
        self.model.setNameFilterDisables(False)
        self.tree.clicked.connect(self.on_file_clicked)
        self.tree.resizeColumnToContents(0)
        self.tree.setColumnWidth(0, 400)
        self.tree.header().setStretchLastSection(True)

        # ===== 右側 GPU Viewer + Label =====
        right_widget = QWidget()
        right_layout = QVBoxLayout()
        right_layout.setContentsMargins(0, 0, 0, 0)
        right_layout.setSpacing(0)
        right_widget.setLayout(right_layout)

        self.filename_label = QLabel(language_wrapper.language_word_dict.get("main_window_current_filename"))
        self.filename_label.setMinimumHeight(16)  # 保證有高度
        self.filename_label.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
        self.filename_label.setAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
        self.filename_label.setWordWrap(False)
        self.filename_label.setTextInteractionFlags(Qt.TextInteractionFlag.TextSelectableByMouse)
        right_layout.addWidget(self.filename_label, stretch=0)

        # GPU Viewer 佔滿剩餘空間
        from Imervue.gpu_image_view.gpu_image_view import GPUImageView
        self.viewer = GPUImageView(main_window=self)
        right_layout.addWidget(self.viewer, stretch=1)

        self.viewer.on_filename_changed = lambda name: self.filename_label.setText(
            language_wrapper.language_word_dict.get("main_window_current_filename_format").format(name=name))

        splitter.addWidget(self.tree)
        splitter.addWidget(right_widget)
        splitter.setStretchFactor(1, 1)
        splitter.setSizes([400, 1000])

        # ===== 選單列 =====
        self.create_menu()

        # ===== Debug =====
        # Debug 模式下自動關閉
        # Auto close in debug mode
        if debug:
            self.debug_timer = QTimer()
            self.debug_timer.setInterval(10000)
            self.debug_timer.timeout.connect(self.debug_close)
            self.debug_timer.start()

    # ==========================
    # 選單
    # ==========================
    def create_menu(self):
        menubar = self.menuBar()

        # ===== 檔案 =====
        file_menu = menubar.addMenu(language_wrapper.language_word_dict.get("main_window_current_file"))

        open_folder_action = file_menu.addAction(language_wrapper.language_word_dict.get("main_window_open_folder"))
        open_folder_action.triggered.connect(self.open_folder)

        # 新增刪除動作
        delete_action = file_menu.addAction(language_wrapper.language_word_dict.get("main_window_remove_undo_stack"))
        delete_action.triggered.connect(lambda: commit_pending_deletions(self.viewer))

        exit_action = file_menu.addAction(language_wrapper.language_word_dict.get("main_window_exit"))
        exit_action.triggered.connect(self.close)

        # ===== Tile Size 選單 =====
        view_menu = menubar.addMenu(language_wrapper.language_word_dict.get("main_window_tile_size"))

        tile_group = QActionGroup(self)
        tile_group.setExclusive(True)

        thumbnail_size = [128, 256, 512, 1024, "None"]

        for size in thumbnail_size:
            if size != "None":
                action = view_menu.addAction(f"{size} x {size}")
            else:
                action = view_menu.addAction(size)
            action.setCheckable(True)

            if size == self.viewer.thumbnail_size:
                action.setChecked(True)

            tile_group.addAction(action)

            action.triggered.connect(
                lambda checked, s=size: self.change_tile_size(s)
            )

        # ===== 語言選單 =====
        build_language_menu(self)
        build_tip_menu(self)

    def change_tile_size(self, size):
        if size != "None":
            self.viewer.thumbnail_size = size
        else:
            self.viewer.thumbnail_size = None

        # 如果目前有載入圖片，重新刷新
        if self.viewer.model.images:
            if len(self.viewer.model.images) > 1:
                self.viewer.clear_tile_grid()
                self.viewer.load_tile_grid_async(image_paths=self.viewer.model.images)
            else:
                load_image(self.viewer.model.images[0], self.viewer)

    # ==========================
    # 開啟資料夾
    # ==========================
    def open_folder(self):
        folder = QFileDialog.getExistingDirectory(
            self, language_wrapper.language_word_dict.get("main_window_select_folder"))

        if folder:
            self.tree.setRootIndex(self.model.index(folder))

    # ==========================
    # 點擊檔案
    # ==========================
    def on_file_clicked(self, index):
        path = self.model.filePath(index)

        # 切換資料夾前先清空
        self.viewer.clear_tile_grid()

        if os.path.isdir(path):
            images = [
                os.path.join(path, f)
                for f in os.listdir(path)
                if f.lower().endswith((".png", ".jpg", ".jpeg", ".bmp", ".tiff"))
            ]
            if images:
                self.viewer.load_tile_grid_async(image_paths=images)
                self.filename_label.setText(
                    language_wrapper.language_word_dict.get("main_window_current_folder_format").format(path=path))
        elif os.path.isfile(path):
            if path.lower().endswith((".png", ".jpg", ".jpeg", ".bmp", ".tiff")):
                load_image(path, self.viewer)
                self.viewer.current_index = 0
                self.viewer.image_list = [path]
                self.filename_label.setText(
                    language_wrapper.language_word_dict.get(
                        "main_window_current_filename_format").format(name=os.path.basename(path)))

    def closeEvent(self, event):
        commit_pending_deletions(self.viewer)
        event.accept()
        write_user_setting()
        super().closeEvent(event)

    @classmethod
    def debug_close(cls) -> None:
        """Debug 模式下強制退出 / Force exit in debug mode"""
        sys.exit(0)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = ImervueMainWindow()
    window.showMaximized()
    sys.exit(app.exec())



class ImageModel:
    def __init__(self):
        self.images: list[str] = []
        self.deleted_history = []

    def set_images(self, paths: list[str]):
        self.images = paths.copy()

    def delete_images(self, paths: list[str]):
        deleted = []
        for p in paths:
            if p in self.images:
                index = self.images.index(p)
                self.images.remove(p)
                deleted.append((p, index))
        return deleted

    def restore_images(self, deleted_items):
        for path, index in sorted(deleted_items, key=lambda x: x[1]):
            self.images.insert(index, path)



from __future__ import annotations

from typing import TYPE_CHECKING

from Imervue.image_type.pyramid import DeepZoomImage
from Imervue.image_type.tile_manager import TileManager

if TYPE_CHECKING:
    from Imervue.gpu_image_view.gpu_image_view import GPUImageView

import os

import imageio
import numpy as np
import rawpy
from PIL import Image


def load_image_file(path, thumbnail=False):
    """
    支援一般圖片 + RAW 檔案
    thumbnail=True 時會優先使用 RAW embedded preview 或 half_size
    回傳 numpy RGBA
    """
    ext = os.path.splitext(path)[1].lower()

    raw_exts = [".cr2", ".nef", ".arw", ".dng", ".raf", ".orf"]

    # ===== RAW 檔案 =====
    if ext in raw_exts:
        with rawpy.imread(path) as raw:

            # ---- Thumbnail 模式 ----
            if thumbnail:
                # 優先讀 embedded preview
                try:
                    thumb = raw.extract_thumb()
                    if thumb.format == rawpy.ThumbFormat.JPEG:
                        img = imageio.v3.imread(thumb.data)
                    elif thumb.format == rawpy.ThumbFormat.BITMAP:
                        img = thumb.data
                    else:
                        raise Exception("No valid embedded preview")
                except Exception:
                    # fallback 用 half_size
                    img = raw.postprocess(
                        half_size=True,
                        use_camera_wb=True,
                        output_bps=8
                    )
            else:
                # ---- 全解析 ----
                img = raw.postprocess(
                    use_camera_wb=True,
                    no_auto_bright=False,
                    output_bps=8
                )

        img_data = img

    # ===== 一般圖片 =====
    else:
        img = Image.open(path).convert("RGBA")
        img_data = np.array(img)

    # ===== 補 alpha =====
    if img_data.shape[2] == 3:
        alpha = np.ones((*img_data.shape[:2], 1), dtype=np.uint8) * 255
        img_data = np.concatenate([img_data, alpha], axis=2)

    return img_data

def load_image(path: str, main_gui: GPUImageView):
    img = Image.open(path).convert("RGBA")
    img_data = np.array(img)
    main_gui.deep_zoom = DeepZoomImage(img_data)
    main_gui.tile_manager = TileManager(main_gui.deep_zoom)
    main_gui.zoom = 1.0
    # 居中
    main_gui.offset_x = (main_gui.width() - img_data.shape[1]) / 2
    main_gui.offset_y = (main_gui.height() - img_data.shape[0]) / 2
    main_gui.update()

def switch_back_to_grid(main_gui: GPUImageView):

    # 1️⃣ 停止 DeepZoom tile manager
    if main_gui.tile_manager:
        main_gui.clear_tile_grid()
        main_gui.tile_manager = None
        main_gui.deep_zoom_mode = False

    main_gui.deep_zoom = None

    main_gui.tile_grid_mode = True

    main_gui.zoom = 1.0
    main_gui.dz_offset_x = 0
    main_gui.dz_offset_y = 0

    main_gui.update()




english_word_dict = {
    # Main window
    "main_window_current_filename": "Current Filename:",
    "main_window_current_filename_format": "Current Filename: {name}",
    "main_window_current_file": "File",
    "main_window_open_folder": "Open Folder",
    "main_window_exit": "Exit",
    "main_window_tile_size": "Thumbnail Size",
    "main_window_select_folder": "Select Folder",
    "main_window_current_folder_format": "Current Folder: {path}",
    "main_window_remove_undo_stack": "Remove temp undo files",
    # Language Menubar
    "menu_bar_language": "Language",
    "language_menu_bar_please_restart_messagebox": "Please restart",
    "language_menu_bar_english": "English",
    "language_menu_bar_traditional_chinese": "Traditional Chinese",
    "language_menu_bar_chinese": "Chinese",
    "language_menu_bar_koren": "Korean",
    "language_menu_bar_japanese": "Japanese",
    # Tip menu
    "main_window_tip_menu": "Instructions",
    "main_window_mouse_tip_menu": "Mouse Control",
    "main_window_keyboard_tip_menu": "Keyboard Control",
    "mouse_control_middle_tip": "You can move using the mouse middle button",
    "mouse_control_left_tip": "Clicking an image with the left mouse button enters Deep zoom mode",
    "mouse_control_right_move_tip": "Hold down the right mouse button to multi-select images in thumbnail mode",
    "keyboard_control_esc_tip": "Press ESC to exit Deep zoom mode",
    "keyboard_control_tile_arrow_up_tip": "In thumbnail mode, use the Up arrow key to move upward",
    "keyboard_control_tile_arrow_down_tip": "In thumbnail mode, use the Down arrow key to move downward",
    "keyboard_control_tile_arrow_left_tip": "In thumbnail mode, use the Left arrow key to move left",
    "keyboard_control_tile_arrow_right_tip": "In thumbnail mode, use the Right arrow key to move right",
    "keyboard_control_delete_tip": "You can use Delete to remove images in both thumbnail and Deep zoom modes",
}


from __future__ import annotations

from typing import TYPE_CHECKING

from PySide6.QtCore import QRectF

if TYPE_CHECKING:
    from Imervue.gpu_image_view.gpu_image_view import GPUImageView

def switch_to_next_image(main_gui: GPUImageView) -> None:
    images = main_gui.model.images

    if not images:
        return

    if main_gui.current_index < len(images) - 1:
        main_gui.current_index += 1
        main_gui.load_deep_zoom_image(images[main_gui.current_index])

    main_gui.update()



def switch_to_previous_image(main_gui: GPUImageView) -> None:
    images = main_gui.model.images

    if not images:
        return

    if main_gui.current_index > 0:
        main_gui.current_index -= 1
        main_gui.load_deep_zoom_image(images[main_gui.current_index])

    main_gui.update()


def select_tiles_in_rect(start_pos, end_pos, main_gui: GPUImageView):
    if start_pos is None or end_pos is None:
        return

    x0, y0 = start_pos.x(), start_pos.y()
    x1, y1 = end_pos.x(), end_pos.y()
    rect = QRectF(min(x0, x1), min(y0, y1), abs(x1 - x0), abs(y1 - y0))

    for tx0, ty0, tx1, ty1, path in main_gui.tile_rects:
        tile_rect = QRectF(tx0, ty0, tx1 - tx0, ty1 - ty0)
        if rect.intersects(tile_rect):
            main_gui.selected_tiles.add(path)

def start_tile_selection(main_gui: GPUImageView):
    if not main_gui.press_pos:
        return

    main_gui.tile_selection_mode = True
    main_gui._drag_selecting = True
    main_gui._drag_start_pos = main_gui.press_pos
    main_gui._drag_end_pos = main_gui.press_pos

    main_gui.update()



from PySide6.QtCore import QRunnable, Signal, QObject
import numpy as np
import os
import rawpy
import imageio
from PIL import Image


class WorkerSignals(QObject):
    finished = Signal(object, str)


class LoadThumbnailWorker(QRunnable):

    def __init__(self, path, size=None):
        super().__init__()
        self.path = path
        self.size = size      # None = 使用原圖尺寸
        self.signals = WorkerSignals()
        self._abort = False

    def abort(self):
        self._abort = True

    def run(self):
        if self._abort:
            return

        try:
            ext = os.path.splitext(self.path)[1].lower()
            raw_exts = [".cr2", ".nef", ".arw", ".dng", ".raf", ".orf"]

            # RAW 圖片
            if ext in raw_exts:
                with rawpy.imread(self.path) as raw:
                    try:
                        thumb = raw.extract_thumb()

                        if thumb.format == rawpy.ThumbFormat.JPEG:
                            img = imageio.imread(thumb.data)

                        elif thumb.format == rawpy.ThumbFormat.BITMAP:
                            img = thumb.data

                        else:
                            raise Exception

                    except Exception:
                        # fallback: 用 half_size 比較安全
                        img = raw.postprocess(
                            half_size=(self.size is not None),
                            use_camera_wb=True,
                            output_bps=8
                        )

                img_data = img

                # 自動限制最大尺寸（避免爆 VRAM）
                if self.size is None:  # 只有使用原圖模式才限制
                    MAX_DIM = 2048

                    h, w = img_data.shape[:2]
                    max_current = max(w, h)

                    if max_current > MAX_DIM:
                        scale = MAX_DIM / max_current
                        new_w = int(w * scale)
                        new_h = int(h * scale)

                        img_pil = Image.fromarray(img_data)
                        img_pil = img_pil.resize(
                            (new_w, new_h),
                            Image.Resampling.LANCZOS
                        )
                        img_data = np.array(img_pil)

                # 如果有指定 size，縮放
                if self.size is not None:
                    img_pil = Image.fromarray(img_data).convert("RGBA")
                    img_pil = img_pil.resize(
                        (self.size, self.size),
                        Image.Resampling.LANCZOS
                    )
                    img_data = np.array(img_pil)

            # 一般圖片
            else:
                img = Image.open(self.path).convert("RGBA")

                if self.size is not None:
                    img = img.resize(
                        (self.size, self.size),
                        Image.Resampling.LANCZOS
                    )

                img_data = np.array(img)

            # 保證有 Alpha
            if img_data.shape[2] == 3:
                alpha = np.ones((*img_data.shape[:2], 1), dtype=np.uint8) * 255
                img_data = np.concatenate([img_data, alpha], axis=2)

            if not self._abort:
                self.signals.finished.emit(img_data, self.path)

        except Exception as e:
            print(f"Thumbnail load failed: {self.path} - {e}")



from __future__ import annotations

import os
from typing import TYPE_CHECKING

from OpenGL.GL import glDeleteTextures

from Imervue.gpu_image_view.images.load_thumbnail_worker import LoadThumbnailWorker

if TYPE_CHECKING:
    from Imervue.gpu_image_view.gpu_image_view import GPUImageView


def delete_current_image(main_gui: GPUImageView):
    images = main_gui.model.images

    if not images or main_gui.current_index >= len(images):
        return

    deleted_index = main_gui.current_index
    path_to_delete = images[deleted_index]

    # ===== 從 model 刪除 =====
    images.pop(deleted_index)

    # ===== 存入 undo stack（統一格式）=====
    main_gui.undo_stack.append({
        "mode": "delete",
        "deleted_paths": [path_to_delete],
        "indices": [deleted_index],
    })

    # ===== 清 GPU texture =====
    tex = main_gui.tile_textures.pop(path_to_delete, None)
    if tex is not None:
        glDeleteTextures([tex])

    # ===== 更新 current_index =====
    if images:
        main_gui.current_index = min(deleted_index, len(images) - 1)
        main_gui.load_deep_zoom_image(images[main_gui.current_index])
    else:
        main_gui.deep_zoom = None
        main_gui.current_index = 0
        main_gui.tile_grid_mode = True

    main_gui.update()


def delete_selected_tiles(main_gui):
    paths = list(main_gui.selected_tiles)
    if not paths:
        return

    images = main_gui.model.images

    deleted_paths = []
    deleted_indices = []

    for path in paths:
        if path in images:
            idx = images.index(path)
            images.remove(path)
            deleted_paths.append(path)
            deleted_indices.append(idx)

    main_gui.undo_stack.append({
        "mode": "delete",
        "deleted_paths": deleted_paths,
        "indices": deleted_indices,
    })

    # GPU
    for path in deleted_paths:
        tex = main_gui.tile_textures.pop(path, None)
        if tex is not None:
            glDeleteTextures([tex])

    # CPU cache
    for path in deleted_paths:
        main_gui.tile_cache.pop(path, None)

    main_gui.selected_tiles.clear()
    main_gui.tile_selection_mode = False
    main_gui.tile_rects.clear()

    main_gui.update()

def undo_delete(main_gui: GPUImageView):
    if not main_gui.undo_stack:
        return

    action = main_gui.undo_stack.pop()

    if action.get("mode") != "delete":
        return

    paths = action["deleted_paths"]
    indices = action["indices"]

    # ===== 依 index 排序插回 =====
    for path, idx in sorted(zip(paths, indices), key=lambda x: x[1]):
        main_gui.model.images.insert(idx, path)

        # ===== 重新載入 thumbnail（tile grid 用）=====
        worker = LoadThumbnailWorker(path, main_gui.thumbnail_size)
        worker.signals.finished.connect(main_gui.add_thumbnail)
        main_gui.thread_pool.start(worker)

    # ===== 如果在 deep zoom 模式，重新載入當前圖 =====
    if main_gui.deep_zoom and main_gui.model.images:
        current_path = main_gui.model.images[main_gui.current_index]
        main_gui.load_deep_zoom_image(current_path)

    main_gui.update()


def clear_thumbnail_cache(main_gui: GPUImageView):
    for tex in main_gui.tile_textures.values():
        glDeleteTextures([tex])
    main_gui.tile_textures.clear()


def commit_pending_deletions(main_gui: GPUImageView):
    for action in main_gui.undo_stack:
        for path in action.get("deleted_paths", []):
            try:
                if os.path.exists(path):
                    os.remove(path)
                    print(f"Permanent delete: {path}")
            except Exception as e:
                print(f"Failed to permanently delete {path}: {e}")

    main_gui.undo_stack.clear()



traditional_chinese_word_dict = {
    "main_window_current_filename": "目前檔名：",
    "main_window_current_filename_format": "目前檔名： {name}",
    "main_window_current_file": "檔案",
    "main_window_open_folder": "開啟資料夾",
    "main_window_exit": "離開",
    "main_window_tile_size": "縮圖大小",
    "main_window_select_folder": "選擇資料夾",
    "main_window_current_folder_format": "目前資料夾： {path}",
    "main_window_remove_undo_stack": "刪除暫存撤銷檔案",
    # Language Menubar
    "menu_bar_language": "語言",
    "language_menu_bar_please_restart_messagebox": "請重新啟動",
    "language_menu_bar_english": "英文",
    "language_menu_bar_traditional_chinese": "繁體中文",
    "language_menu_bar_chinese": "簡體中文",
    "language_menu_bar_koren": "韓文",
    "language_menu_bar_japanese": "日文",
    # Tip menu
    "main_window_tip_menu": "使用說明",
    "main_window_mouse_tip_menu": "滑鼠操作",
    "main_window_keyboard_tip_menu": "鍵盤操作",
    "mouse_control_middle_tip": "可以使用滑鼠中鍵移動",
    "mouse_control_left_tip": "滑鼠左鍵點擊圖片可以進入 Deep zoom 模式",
    "mouse_control_right_move_tip": "滑鼠右鍵按住可以在縮圖模式下複選圖片",
    "keyboard_control_esc_tip": "Deep zoom 模式下可以按 ESC 退出",
    "keyboard_control_tile_arrow_up_tip": "縮圖模式下可以使用方向鍵的上來往上移動",
    "keyboard_control_tile_arrow_down_tip": "縮圖模式下可以使用方向鍵的下來往下移動",
    "keyboard_control_tile_arrow_left_tip": "縮圖模式下可以使用方向鍵的左來往左移動",
    "keyboard_control_tile_arrow_right_tip": "縮圖模式下可以使用方向鍵的右來往右移動",
    "keyboard_control_delete_tip": "縮圖或 Deep zoom 模式都可以使用 Delete 來刪除圖片",
}


from __future__ import annotations

import os
from typing import TYPE_CHECKING

from Imervue.gpu_image_view.actions.delete import delete_current_image, delete_selected_tiles, undo_delete
from Imervue.gpu_image_view.actions.select import switch_to_next_image, switch_to_previous_image, select_tiles_in_rect, \
    start_tile_selection
from Imervue.gpu_image_view.images.image_loader import load_image_file
from Imervue.gpu_image_view.images.image_model import ImageModel
from Imervue.gpu_image_view.images.load_thumbnail_worker import LoadThumbnailWorker

if TYPE_CHECKING:
    from Imervue.Imervue_main_window import ImervueMainWindow

import numpy as np
from OpenGL.GL import *
from PySide6.QtCore import QThreadPool, QMutex, QTimer, Qt
from PySide6.QtOpenGLWidgets import QOpenGLWidget

from Imervue.image_type.pyramid import DeepZoomImage
from Imervue.image_type.tile_manager import TileManager


class GPUImageView(QOpenGLWidget):
    def __init__(self, main_window: ImervueMainWindow):
        super().__init__()

        self.main_window = main_window

        # ===== 模式 =====
        self.tile_grid_mode = False
        self.deep_zoom_mode = False
        self.selected_image_path = None

        # ===== Undo =====
        self.undo_stack = []

        # ===== Tile Grid =====

        self.tile_rects = []  # 用來存每個 tile 的 rectangle
        self.grid_offset_x = 0
        self.grid_offset_y = 0
        self.tile_scale = 1.0
        self.tile_textures = {}
        self.tile_cache = {}  # path -> img_data

        # ===== DeepZoom =====
        self.zoom = 1.0
        self.dz_offset_x = 0
        self.dz_offset_y = 0
        self.last_pos = None
        self.tile_manager = None
        self.deep_zoom = None
        self._saved_tile_state = None

        # ===== offset =====
        self.offset_x = None
        self.offset_y = None

        # ===== 圖片切換控制 =====
        self.model = ImageModel()
        self.model.images = []  # 所有圖片路徑
        self.current_index = 0
        self.on_filename_changed = None
        self.deep_zoom_tile_size = 512
        self.thumbnail_size = 512

        # ===== Tile Grid 選取模式 =====
        self.tile_selection_mode = False  # 是否在選取模式
        self.selected_tiles = set()  # 已選取的 tile path
        self.long_press_threshold = 500  # 長按進入選取模式的毫秒
        self._press_timer = None
        self._drag_selecting = False  # 是否正在拖曳框選
        self._drag_start_pos = None
        self._drag_end_pos = None

        # ===== Mouse =====
        self._middle_dragging = None
        self.press_pos = None

        # ===== Thread =====
        self.thread_pool = QThreadPool.globalInstance()
        self.grid_mutex = QMutex()  # 保護 tile_grid 併發更新
        self.active_tile_workers = []  # 用來追蹤 Tile Grid 載入 worker
        self.active_deep_zoom_worker = None

        # ===== Focus ======
        self.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
        self.setFocus()

    # ===========================
    # OpenGL 初始化
    # ===========================
    def initializeGL(self):
        glEnable(GL_TEXTURE_2D)
        glClearColor(0.1, 0.1, 0.1, 1)

    def resizeGL(self, w, h):
        glViewport(0, 0, w, h)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(0, w, h, 0, -1, 1)
        glMatrixMode(GL_MODELVIEW)

    # ===========================
    # 繪製
    # ===========================
    def paintGL(self):
        glClear(GL_COLOR_BUFFER_BIT)

        # ===== Tile Grid Mode =====
        if self.tile_grid_mode:
            self.paint_tile_grid()
        # ===== DeepZoom Mode =====
        elif self.deep_zoom:
            self.paint_deep_zoom()

    # ---------------------------
    # Tile Grid Lazy Render
    # ---------------------------
    def paint_tile_grid(self):

        glLoadIdentity()

        images = self.model.images
        if self.thumbnail_size is not None:
            base_tile = self.thumbnail_size
        elif self.tile_cache:
            # 用第一張圖實際寬度當排版基準
            first_img = next(iter(self.tile_cache.values()))
            base_tile = first_img.shape[1]
        else:
            base_tile = 256

        scaled_tile = base_tile * self.tile_scale
        cols = max(1, int(self.width() // scaled_tile))
        self.tile_rects = []

        for i, path in enumerate(images):

            # 沒載入完成就跳過
            if path not in self.tile_cache:
                continue

            img_data = self.tile_cache[path]

            row = i // cols
            col = i % cols

            x0 = col * scaled_tile + self.grid_offset_x
            y0 = row * scaled_tile + self.grid_offset_y
            x1 = x0 + img_data.shape[1] * self.tile_scale
            y1 = y0 + img_data.shape[0] * self.tile_scale

            # Viewport 裁切
            if x1 < 0 or x0 > self.width() or y1 < 0 or y0 > self.height():
                continue

            self.tile_rects.append((x0, y0, x1, y1, path))

            # ===== GPU texture =====
            if path not in self.tile_textures:
                tex = glGenTextures(1)
                glBindTexture(GL_TEXTURE_2D, tex)
                glPixelStorei(GL_UNPACK_ALIGNMENT, 1)

                glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
                             img_data.shape[1], img_data.shape[0], 0,
                             GL_RGBA, GL_UNSIGNED_BYTE, img_data)

                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
                glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)

                self.tile_textures[path] = tex

            glColor4f(1, 1, 1, 1)
            glBindTexture(GL_TEXTURE_2D, self.tile_textures[path])

            glBegin(GL_QUADS)
            glTexCoord2f(0, 1)
            glVertex2f(x0, y1)
            glTexCoord2f(1, 1)
            glVertex2f(x1, y1)
            glTexCoord2f(1, 0)
            glVertex2f(x1, y0)
            glTexCoord2f(0, 0)
            glVertex2f(x0, y0)
            glEnd()

        # Tile selection overlay
        if self.tile_selection_mode:
            glDisable(GL_TEXTURE_2D)
            glEnable(GL_BLEND)
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
            glLineWidth(4)

            for x0, y0, x1, y1, path in self.tile_rects:
                if path in self.selected_tiles:
                    # 藍色粗邊框
                    glColor4f(0.18, 0.5, 1.0, 1.0)
                    glBegin(GL_LINE_LOOP)
                    glVertex2f(x0, y0)
                    glVertex2f(x1, y0)
                    glVertex2f(x1, y1)
                    glVertex2f(x0, y1)
                    glEnd()

                    # 右上藍色圓 + 勾
                    circle_radius = 9
                    cx = x1 - 12
                    cy = y0 + 12
                    glBegin(GL_TRIANGLE_FAN)
                    glVertex2f(cx, cy)
                    for i in range(33):
                        angle = i * 2.0 * 3.1415926 / 32
                        glVertex2f(cx + circle_radius * np.cos(angle),
                                   cy + circle_radius * np.sin(angle))
                    glEnd()
                    glColor4f(1.0, 1.0, 1.0, 1.0)
                    glLineWidth(2.5)
                    glBegin(GL_LINES)
                    glVertex2f(cx - 4, cy)
                    glVertex2f(cx - 1, cy + 3)
                    glVertex2f(cx - 1, cy + 3)
                    glVertex2f(cx + 5, cy - 4)
                    glEnd()
                    glLineWidth(4)

            glDisable(GL_BLEND)
            glEnable(GL_TEXTURE_2D)
            glColor4f(1, 1, 1, 1)
            glLineWidth(1)

        if self._drag_selecting and self._drag_start_pos and self._drag_end_pos:
            x0, y0 = self._drag_start_pos.x(), self._drag_start_pos.y()
            x1, y1 = self._drag_end_pos.x(), self._drag_end_pos.y()

            left = min(x0, x1)
            right = max(x0, x1)
            top = min(y0, y1)
            bottom = max(y0, y1)

            # ===== 明確設定狀態（不要 push/pop）=====
            glDisable(GL_TEXTURE_2D)

            glEnable(GL_BLEND)
            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

            # 淡藍填充
            glColor4f(0.18, 0.5, 1.0, 0.08)
            glBegin(GL_QUADS)
            glVertex2f(left, top)
            glVertex2f(right, top)
            glVertex2f(right, bottom)
            glVertex2f(left, bottom)
            glEnd()

            # 藍色粗框
            glColor4f(0.18, 0.5, 1.0, 1.0)
            glLineWidth(3)
            glBegin(GL_LINE_LOOP)
            glVertex2f(left, top)
            glVertex2f(right, top)
            glVertex2f(right, bottom)
            glVertex2f(left, bottom)
            glEnd()

            # ===== 恢復狀態 =====
            glDisable(GL_BLEND)
            glEnable(GL_TEXTURE_2D)
            glColor4f(1, 1, 1, 1)

    # ---------------------------
    # DeepZoom Lazy Render
    # ---------------------------
    def paint_deep_zoom(self):
        if not self.deep_zoom:
            return

        level, _ = self.deep_zoom.get_level(self.zoom)
        level_image = self.deep_zoom.levels[level]
        base_image = self.deep_zoom.levels[0]
        scale_x = self.zoom * (level_image.shape[1] / base_image.shape[1])
        scale_y = self.zoom * (level_image.shape[0] / base_image.shape[0])

        glLoadIdentity()
        glScalef(scale_x, scale_y, 1)
        glTranslatef(self.dz_offset_x / scale_x, self.dz_offset_y / scale_y, 0)

        # 計算 viewport 內 tiles
        tile_size = self.deep_zoom_tile_size
        h, w = level_image.shape[:2]

        left = -self.dz_offset_x / scale_x
        top = -self.dz_offset_y / scale_y
        right = left + self.width() / scale_x
        bottom = top + self.height() / scale_y

        tx0 = int(left // tile_size)
        tx1 = int(right // tile_size)
        ty0 = int(top // tile_size)
        ty1 = int(bottom // tile_size)

        for tx in range(tx0, tx1 + 1):
            for ty in range(ty0, ty1 + 1):
                if 0 <= tx * tile_size < w and 0 <= ty * tile_size < h:
                    tex = self.tile_manager.get_tile(level, tx, ty, tile_size)
                    if tex is None:
                        self.tile_manager.load_tile_async(level, tx, ty, tile_size)
                        continue
                    glBindTexture(GL_TEXTURE_2D, tex)
                    tile_w = min(tile_size, w - tx * tile_size)
                    tile_h = min(tile_size, h - ty * tile_size)
                    x = tx * tile_size
                    y = ty * tile_size
                    glBegin(GL_QUADS)
                    glTexCoord2f(0, 1)
                    glVertex2f(x, y + tile_h)
                    glTexCoord2f(1, 1)
                    glVertex2f(x + tile_w, y + tile_h)
                    glTexCoord2f(1, 0)
                    glVertex2f(x + tile_w, y)
                    glTexCoord2f(0, 0)
                    glVertex2f(x, y)
                    glEnd()

    # ===========================
    # 非同步載入縮圖
    # ===========================
    def load_tile_grid_async(self, image_paths):

        self.model.set_images(image_paths)

        self.tile_cache.clear()
        self.tile_textures.clear()

        self.tile_grid_mode = True
        self.deep_zoom = None

        for path in image_paths:
            worker = LoadThumbnailWorker(path, self.thumbnail_size)
            worker.signals.finished.connect(self.add_thumbnail)
            self.thread_pool.start(worker)

        self.update()

    def load_deep_zoom_image(self, path):
        img_data = load_image_file(path, thumbnail=False)
        self.deep_zoom = DeepZoomImage(img_data)
        self.tile_manager = TileManager(self.deep_zoom)

        self.zoom = 1.0
        self.dz_offset_x = 0
        self.dz_offset_y = 0

        self.update()

        if self.on_filename_changed:
            self.on_filename_changed(os.path.basename(path))

    def clear_tile_grid(self):
        self.tile_grid_mode = False
        self.tile_rects = []

        # 重置偏移量
        self.grid_offset_x = 0
        self.grid_offset_y = 0

        self.update()

    def add_thumbnail(self, img_data, path):

        # 如果已被刪除，不加入
        if path not in self.model.images:
            return

        self.tile_cache[path] = img_data
        self.update()

    # ===========================
    # Event
    # ===========================
    def wheelEvent(self, event):
        delta = event.angleDelta().y()
        factor = 1.1 if delta > 0 else 0.9

        if self.tile_grid_mode:
            self.tile_scale *= factor
            self.tile_scale = max(0.1, min(5.0, self.tile_scale))
            self.update()

        elif self.deep_zoom:
            old_zoom = self.zoom
            self.zoom *= factor

            cx, cy = self.width() / 2, self.height() / 2

            self.dz_offset_x = cx - (cx - self.dz_offset_x) * (self.zoom / old_zoom)
            self.dz_offset_y = cy - (cy - self.dz_offset_y) * (self.zoom / old_zoom)

            self.update()

    def mousePressEvent(self, event):
        self.last_pos = event.position()

        # ===== 中鍵 → 進入拖動模式 =====
        if event.button() == Qt.MouseButton.MiddleButton:
            self._middle_dragging = True
            return
        elif event.button() == Qt.MouseButton.RightButton:
            if self.tile_grid_mode:
                self.press_pos = event.position()

                self._press_timer = QTimer(self)
                self._press_timer.setSingleShot(True)
                self._press_timer.timeout.connect(lambda: start_tile_selection(self))
                self._press_timer.start(self.long_press_threshold)


        # ===== 左鍵 =====
        if event.button() == Qt.MouseButton.LeftButton:
            if self.tile_grid_mode:
                mx, my = event.position().x(), event.position().y()
                for x0, y0, x1, y1, path in self.tile_rects:
                    if x0 <= mx <= x1 and y0 <= my <= y1:
                        # 先存 Tile Grid 狀態
                        self._saved_tile_state = {
                            "grid_offset_x": self.grid_offset_x,
                            "grid_offset_y": self.grid_offset_y,
                            "tile_scale": self.tile_scale,
                        }

                        self.current_index = self.model.images.index(path)
                        self.tile_grid_mode = False
                        self.load_deep_zoom_image(path)
                        self.update()
                        return
        super().mousePressEvent(event)


    def mouseMoveEvent(self, event):
        delta = event.position() - self.last_pos
        self.last_pos = event.position()

        # ===== 中鍵拖動 =====
        if self._middle_dragging:
            if self.tile_grid_mode:
                self.grid_offset_x += delta.x()
                self.grid_offset_y += delta.y()
            elif self.deep_zoom:
                self.dz_offset_x += delta.x()
                self.dz_offset_y += delta.y()

            self.update()
            return

        # ===== 左鍵框選 =====
        if self.tile_grid_mode and self._drag_selecting:
            self._drag_end_pos = event.position()
            self.update()

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.MouseButton.MiddleButton:
            self._middle_dragging = False
            return

        if self._press_timer:
            self._press_timer.stop()
            self._press_timer = None

        self.press_pos = None

        if self.tile_grid_mode:
            mx, my = event.position().x(), event.position().y()

            if self._drag_selecting:
                # 拖曳框選完成
                select_tiles_in_rect(self._drag_start_pos, self._drag_end_pos, self)
                self._drag_selecting = False
                self._drag_start_pos = None
                self._drag_end_pos = None
                self.update()
                return

            clicked_any_tile = False

            # 單點選擇
            for x0, y0, x1, y1, path in self.tile_rects:
                if x0 <= mx <= x1 and y0 <= my <= y1:
                    if self.tile_selection_mode:
                        if path in self.selected_tiles:
                            self.selected_tiles.remove(path)
                        else:
                            self.selected_tiles.add(path)
                        self.update()
                        return
                    else:
                        self.current_index = self.model.images.index(path)
                        self.tile_grid_mode = False
                        self.load_deep_zoom_image(path)
                        self.update()
                        return
            # 如果在選取模式，且點擊空白 → 退出選取模式
            if self.tile_selection_mode and not clicked_any_tile:
                self.tile_selection_mode = False
                self.selected_tiles.clear()
                self.update()

    def keyPressEvent(self, event):
        key = event.key()
        modifiers = event.modifiers()
        # Undo
        if modifiers & Qt.KeyboardModifier.ControlModifier:
            if key == Qt.Key.Key_Z:
                undo_delete(main_gui=self)
                return

        step = self.thumbnail_size  # 每次偏移量，可根據需求調整
        if step is None:
            step = 1024
        fine_step = int(step / 2)  # 精細調整

        # 如果按住 Shift 做精細移動
        if modifiers & Qt.KeyboardModifier.ShiftModifier:
            move_step = fine_step
        else:
            move_step = step

        # ===== Tile Grid 模式 =====
        if self.tile_grid_mode:
            if key == Qt.Key.Key_Up:
                self.grid_offset_y += move_step
                self.update()
                return
            elif key == Qt.Key.Key_Down:
                self.grid_offset_y -= move_step
                self.update()
                return
            elif key == Qt.Key.Key_Left:
                self.grid_offset_x += move_step
                self.update()
                return
            elif key == Qt.Key.Key_Right:
                self.grid_offset_x -= move_step
                self.update()
                return

        if key == Qt.Key.Key_Escape:
            if self.deep_zoom:
                # 回到原本 Tile Grid
                self.deep_zoom = None
                self.tile_grid_mode = True
                if self._saved_tile_state:
                    self.grid_offset_x = self._saved_tile_state["grid_offset_x"]
                    self.grid_offset_y = self._saved_tile_state["grid_offset_y"]
                    self.tile_scale = self._saved_tile_state["tile_scale"]
                    self._saved_tile_state = None

                self.update()
                return

        # Tile Grid 刪除
        if self.tile_grid_mode and self.tile_selection_mode:
            if key == Qt.Key.Key_Delete:
                delete_selected_tiles(main_gui=self)
                return

        # DeepZoom
        if self.deep_zoom:
            if key == Qt.Key.Key_Right:
                switch_to_next_image(main_gui=self)
                return
            elif key == Qt.Key.Key_Left:
                switch_to_previous_image(main_gui=self)
                return
            elif key == Qt.Key.Key_Delete:
                delete_current_image(main_gui=self)
                return



chinese_word_dict = {
    "main_window_current_filename": "当前文件名：",
    "main_window_current_filename_format": "当前文件名：{name}",
    "main_window_current_file": "文件",
    "main_window_open_folder": "打开文件夹",
    "main_window_exit": "退出",
    "main_window_tile_size": "缩略图大小",
    "main_window_select_folder": "选择文件夹",
    "main_window_current_folder_format": "当前文件夹：{path}",
    "main_window_remove_undo_stack": "Remove temp undo files",
    # Language Menubar
    "menu_bar_language": "语言",
    "language_menu_bar_please_restart_messagebox": "请重新启动",
    "language_menu_bar_english": "英语",
    "language_menu_bar_traditional_chinese": "繁体中文",
    "language_menu_bar_chinese": "简体中文",
    "language_menu_bar_koren": "韩语",
    "language_menu_bar_japanese": "日语",
    # Tip menu
    "main_window_tip_menu": "使用说明",
    "main_window_mouse_tip_menu": "鼠标操作",
    "main_window_keyboard_tip_menu": "键盘操作",
    "mouse_control_middle_tip": "可以使用鼠标中键移动",
    "mouse_control_left_tip": "鼠标左键点击图片可以进入 Deep zoom 模式",
    "mouse_control_right_move_tip": "按住鼠标右键可以在缩略图模式下多选图片",
    "keyboard_control_esc_tip": "Deep zoom 模式下可以按 ESC 退出",
    "keyboard_control_tile_arrow_up_tip": "缩略图模式下可以使用方向键的上来向上移动",
    "keyboard_control_tile_arrow_down_tip": "缩略图模式下可以使用方向键的下来向下移动",
    "keyboard_control_tile_arrow_left_tip": "缩略图模式下可以使用方向键的左来向左移动",
    "keyboard_control_tile_arrow_right_tip": "缩略图模式下可以使用方向键的右来向右移动",
    "keyboard_control_delete_tip": "缩略图或 Deep zoom 模式都可以使用 Delete 来删除图片",

}



import argparse
import os
import sys

from PySide6.QtWidgets import QApplication

from Imervue.Imervue_main_window import ImervueMainWindow

def parse_args():
    parser = argparse.ArgumentParser(description="Start Imervue Application")

    parser.add_argument(
        "--debug",
        action="store_true",
        help="Enable debug mode"
    )
    parser.add_argument(
        "--software_opengl",
        action="store_true",
        help="Enable software opengl"
    )

    return parser.parse_args()

if __name__ == "__main__":
    # 解析參數
    args = parse_args()

    if args.debug:
        debug = True
    else:
        debug = False

    if args.software_opengl:
        os.environ["QT_OPENGL"] = "software"

    app = QApplication(sys.argv)
    window = ImervueMainWindow(debug=debug)
    window.showMaximized()
    sys.exit(app.exec())


from __future__ import annotations

from typing import TYPE_CHECKING

from Imervue.multi_language.language_wrapper import language_wrapper

if TYPE_CHECKING:
    from Imervue.Imervue_main_window import ImervueMainWindow


def build_tip_menu(ui_we_want_to_set: ImervueMainWindow):
    # ===== 主 Tip 選單 =====
    tip_menu = ui_we_want_to_set.menuBar().addMenu(
        language_wrapper.language_word_dict.get("main_window_tip_menu", "Tip")
    )

    # ==========================
    # 滑鼠說明子選單
    # ==========================
    mouse_menu = tip_menu.addMenu(
        language_wrapper.language_word_dict.get("main_window_mouse_tip_menu", "Mouse")
    )

    mouse_keys = [
        "mouse_control_middle_tip",
        "mouse_control_left_tip",
        "mouse_control_right_move_tip",
    ]

    for key in mouse_keys:
        text = language_wrapper.language_word_dict.get(key)
        action = mouse_menu.addAction(text)
        action.setEnabled(False)

    # 主選單分隔線
    tip_menu.addSeparator()

    # ==========================
    # 鍵盤說明子選單
    # ==========================
    keyboard_menu = tip_menu.addMenu(
        language_wrapper.language_word_dict.get("main_window_keyboard_tip_menu", "Keyboard")
    )

    keyboard_keys = [
        "keyboard_control_esc_tip",
        "keyboard_control_tile_arrow_up_tip",
        "keyboard_control_tile_arrow_down_tip",
        "keyboard_control_tile_arrow_left_tip",
        "keyboard_control_tile_arrow_right_tip",
        "keyboard_control_delete_tip",
    ]

    for i, key in enumerate(keyboard_keys):
        text = language_wrapper.language_word_dict.get(key)
        action = keyboard_menu.addAction(text)
        action.setEnabled(False)

        # 在 ESC 和方向鍵之間加分隔線（可選）
        if key == "keyboard_control_esc_tip":
            keyboard_menu.addSeparator()



japanese_word_dict = {
    "main_window_current_filename": "現在のファイル名:",
    "main_window_current_filename_format": "現在のファイル名: {name}",
    "main_window_current_file": "ファイル",
    "main_window_open_folder": "フォルダーを開く",
    "main_window_exit": "終了",
    "main_window_tile_size": "サムネイルサイズ",
    "main_window_select_folder": "フォルダーを選択",
    "main_window_current_folder_format": "現在のフォルダー: {path}",
    "main_window_remove_undo_stack": "一時的な取り消しファイルを削除",
    # Language Menubar
    "menu_bar_language": "言語",
    "language_menu_bar_please_restart_messagebox": "再起動してください",
    "language_menu_bar_english": "英語",
    "language_menu_bar_traditional_chinese": "繁体字中国語",
    "language_menu_bar_chinese": "簡体字中国語",
    "language_menu_bar_koren": "韓国語",
    "language_menu_bar_japanese": "日本語",
    # Tip menu
    "main_window_tip_menu": "使用説明",
    "mouse_control_middle_tip": "マウスの中ボタンで移動できます",
    "main_window_mouse_tip_menu": "マウス操作",
    "main_window_keyboard_tip_menu": "キーボード操作",
    "mouse_control_left_tip": "マウス左ボタンで画像をクリックすると Deep zoom モードに入ります",
    "mouse_control_right_move_tip": "マウスの右ボタンを押したままにすると、サムネイルモードで複数の画像を選択できます",
    "keyboard_control_esc_tip": "Deep zoom モードでは ESC キーで終了できます",
    "keyboard_control_tile_arrow_up_tip": "サムネイルモードでは矢印キーの上で上に移動できます",
    "keyboard_control_tile_arrow_down_tip": "サムネイルモードでは矢印キーの下で下に移動できます",
    "keyboard_control_tile_arrow_left_tip": "サムネイルモードでは矢印キーの左で左に移動できます",
    "keyboard_control_tile_arrow_right_tip": "サムネイルモードでは矢印キーの右で右に移動できます",
    "keyboard_control_delete_tip": "サムネイルモードや Deep zoom モードでは Delete キーで画像を削除できます",
}
